/* 
 * contrast_brightness.h 
 * version 1.1
 * author Felix Burkhardt
 * last change:  May 10 23:46:34 MEST 1998
 *
 * Routinen fue Kontrast- und Brightness-Veraenderungen.
 *
 * Welche Kanaele veraendert werden sollen, wird durch den
 * Parameter flag angegeben, wobei die Bits jewils fuer
 * R, G, B oder Alpha stehen. (also flag=7 waere RGB, flag=2 nur G)
 *
 * Die Routinen nehem einen Pointer auf Beschreibungstukturen
 * fuer die Anderungen
 * Jede Struktur besteht aus Reihenangabe und Veraenderungsfaktor
 * Die Strukturen MUESSEN nach aufsteigenden Reihen sortiert sein
 * Der Faktor ist um 0.5 skaliert (zwischen 0 and 1).
 *
 * Um den Kontrast zu veraendern wird der Mittelwert des Kanals
 * berechnet und als Kontrastgrenze verwendet.
 */


#ifndef _CONTRAST_BRIGHTNESS_H_
#define _CONTRAST_BRIGHTNESS_H_

#include "pixdef.h"
#include "pixutil.h"

typedef struct changeDescriptor *changeDescriptorP;

struct changeDescriptor {
  int row;            /* which row to change                       */
  double val;         /* which factor to change                    */
};

pixval *changeContrast(
	pixval *pix_array_in, 		/* inputPPM                */
	int width, int height,
	int ChangeDescrNum,      /* number of change-descriptors   */
	changeDescriptorP cDPointer, /* change-descriptors         */
	int flag);			/* flag for channels       */
pixval *changeBrightness(
	pixval *pix_array_in, 		/* inputPPM                */
	int width, int height,
	int ChangeDescrNum,      /* number of change-descriptors   */
	changeDescriptorP cDPointer, /* change-descriptors         */
	int flag);

#endif /*_CONTRAST_BRIGHTNESS_H_*/
