
/* test-programm fuer Kontrast und Brightness-Berechnung 
 * @param input-file
 * @param output-file
 * @param minrow - which row to start the change
 * @param maxrow - which row to end the change
 * @param method -  -c or -b for contrast- or brightness-change
 * @param len - how many change-descriptors follow?
 * @param len number of change-descriptors, each consisting
 *        of row and value; they should be sortet!
 */

#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "pixio.h"
#include "pixdef.h"
#include "contrast_brightness.h"

#define STARTTIMER {double ttx;ttx=startTimer();
#define STOPTIMER(f) ttx=stopTimer(ttx);fprintf(stderr,f,ttx);}

double 
startTimer ()
{
      struct timeval tp;
      struct timezone tzp;

      gettimeofday (&tp, &tzp);
      return tp.tv_sec + tp.tv_usec * 1.e-6;
}

double 
stopTimer (double t)
{
      struct timeval tp;
      struct timezone tzp;

      gettimeofday (&tp, &tzp);
      return tp.tv_sec + tp.tv_usec * 1.e-6 - t;
}

void 
usage ()
{
      printf ("        usage: \n "
	      " contrast_brightness_test\n"
	      " <input-file> <output-file>\n"
	      " <method: \"-c\" | \"-b\"> - change contrast or brightness\n"
	      " <flag> - which channels to change (eg flag=7 for RGB\n"
	      " <cdNum> - number of change descriptors\n"
	      " <*change-descriptor> - 2 values: row, value\n");
      exit (-1);
}

int 
main (int argc, char **argv)
{
      FILE *ifp, *ofp;
      int height, width;
      pixval maxval;
      pixval *pix_array_in, *pix_array_out;
      char method[2];
      int len, i, flag;
      changeDescriptorP cDPointer;

      /* check the arguments                                                  */

      if (argc < 8)
	    usage ();

      if ((ifp = fopen (argv[1], "r")) == NULL) {
	    perror (argv[1]);
	    exit (-1);
      }
      pix_array_in = read_pbm (ifp, &width, &height, &maxval);
      fclose (ifp);

      if ((ofp = fopen (argv[2], "w")) == NULL) {
	    perror (argv[2]);
	    exit (-1);
      }

      strcpy (method, argv[3]);

      flag = atoi (argv[4]);

      len = atoi (argv[5]);

      if (argc != 6 + len * 2) {
	    fprintf (stderr, "probably error in change-descriptors!\n");
	    usage ();
      }

      cDPointer = malloc (len * sizeof (struct changeDescriptor));

      for (i = 0; i < len; i++) {
	    cDPointer[i].row = atoi (argv[6 + i * 2]);
	    cDPointer[i].val = (double) atof (argv[7 + i * 2]);
      }

      /* change Contrast                                                       */
      if (strcmp (method, "-c") == 0) {
	    STARTTIMER;
	    pix_array_out = changeContrast (pix_array_in, width, height,
					    len, cDPointer, flag);
	    STOPTIMER ("time used: %.2f \n");

      }
      /* change Brightness */
      else if (strcmp (method, "-b") == 0) {
	    STARTTIMER;
	    pix_array_out = changeBrightness (pix_array_in, width, height,
					      len, cDPointer, flag);
	    STOPTIMER ("time used: %.2f \n");

      }
      else
	    usage ();

      write_pbm (ofp, pix_array_out, width, height, MAXVAL);
      fclose (ofp);

      free (pix_array_out);
      free (pix_array_in);

      return 0;
}
