#include "pixio.h"

static int pm_getint(FILE *ifp);
static char pm_getc(FILE *ifp);
static unsigned char pm_getrawbyte(FILE *file);

extern unsigned char *read_pbm(FILE *ifp, int *cols, int *rows, unsigned char *maxval) {
  unsigned char *pixels;
  int i;
  char MagicNumber[3];

  /* Magic Number lesen */
  MagicNumber[0] = pm_getc(ifp); 
  MagicNumber[1] = pm_getc(ifp);
  MagicNumber[2] = '\0';
  
  *cols = pm_getint(ifp);
  *rows = pm_getint(ifp);

  *maxval = pm_getint(ifp);

  pixels = malloc(*cols * *rows * 4);
  
  if(MagicNumber[1]=='3') 
    for(i=0; i < *cols * *rows * 4; i++)
      if(((i+1)%4)==0)
	pixels[i] = (unsigned char) 0;
      else
	pixels[i] = pm_getint(ifp);
  else if(MagicNumber[1]=='6')
    for(i=0; i < *cols * *rows * 4; i++)
      if(((i+1)%4)==0)
	pixels[i] = (unsigned char) 0;
      else
	pixels[i] = pm_getrawbyte(ifp);
  else
    pm_error("Kein PBM-Format!");
  return pixels;
}
extern unsigned char *read_pgm(FILE *ifp, int *cols, int *rows, unsigned char *maxval) {
  unsigned char *pixels;
  int i;
  char MagicNumber[3];

  /* Magic Number lesen */
  MagicNumber[0] = pm_getc(ifp); 
  MagicNumber[1] = pm_getc(ifp);
  MagicNumber[2] = '\0';
  
  *cols = pm_getint(ifp);
  *rows = pm_getint(ifp);

  *maxval = pm_getint(ifp);

  pixels = malloc(*cols * *rows);
  
  if(MagicNumber[1]=='2')     
    for(i=0; i < *cols * *rows; i++)
      pixels[i] = pm_getint(ifp);
  else if(MagicNumber[1]=='5')
    for(i=0; i < *cols * *rows; i++)
      pixels[i] = pm_getrawbyte(ifp);
  else
    pm_error("Kein PGM-Format!");
  return pixels;
}
extern unsigned char *read_rgba(FILE *ifp, int *cols, int *rows, unsigned char *maxval) {
  unsigned char *pixels;
  int i;
  char MagicNumber[3];
  
  /* Magic Number lesen */
  MagicNumber[0] = pm_getc(ifp); 
  MagicNumber[1] = pm_getc(ifp);
  MagicNumber[2] = '\0';
  
  *cols = pm_getint(ifp);
  *rows = pm_getint(ifp);

  *maxval = pm_getint(ifp);
  
  pixels = malloc(*cols * *rows * 4);
  
  if(MagicNumber[1]=='7') 
    for(i=0; i < *cols * *rows * 4; i++)
      pixels[i] = pm_getint(ifp);
  else if(MagicNumber[1]=='8')
    for(i=0; i < *cols * *rows * 4; i++)
      pixels[i] = pm_getrawbyte(ifp);  
  else
    pm_error("Kein PMA-Format!");
  return pixels;
}

extern void pm_error(char *reason) {
  fprintf(stderr, "%s\n", reason);
  exit(1);
}

extern void write_pbm(FILE *ofp, unsigned char *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P3\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows * 4; i++)
    if((i+1)%4==0) ;
    else 
      fprintf(ofp, "%d ", pixels[i]);
}
extern void write_pbm_raw(FILE *ofp, pixval *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P6\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows * CHANNELS; i++)
    if((i+1)%4==0) ;
    else
      fputc(pixels[i], ofp);
}
extern void write_pgm(FILE *ofp, unsigned char *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P2\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows; i++)
    fprintf(ofp, "%d ", pixels[i]);
}
extern void write_pgm_raw(FILE *ofp, pixval *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P5\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows; i++)
    fputc(pixels[i], ofp);
}
extern void write_rgba(FILE *ofp, unsigned char *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P7\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows * 4; i++)
    
    fprintf(ofp, "%d ", pixels[i]);
}
extern void write_rgba_raw(FILE *ofp, unsigned char *pixels, int cols, int rows, unsigned char maxval) {
  int i;
  fprintf(ofp, "P8\n");
  fprintf(ofp, "%d %d\n", cols, rows);
  fprintf(ofp, "%d\n", maxval);
  for(i=0; i < cols * rows * CHANNELS; i++)
    fputc(pixels[i], ofp);
}
static int pm_getint(FILE *ifp) {
  register char ch;
  register int i;
  
  do {
    ch = pm_getc(ifp);
  }
  while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );
  
  if ( ch < '0' || ch > '9' )
    pm_error("junk in file where an integer should be");
  i = 0;
  do {
    i = i * 10 + ch - '0';
    ch = pm_getc(ifp);
  }
  while ( ch >= '0' && ch <= '9' );
  
  return i;
}

static char pm_getc(FILE *ifp) {
  register int ich;
  register char ch;

  if((ich = getc(ifp)) == EOF || ifp == NULL)
    pm_error("EOF / read error");
  ch = (char) ich;
  if(ch=='#') {
    do {
      ich = getc(ifp);
      if ( ich == EOF )
	pm_error("EOF / read error");
      ch = (char) ich;
    }
    while(ch!='\n' && ch!='\r');
  }
  return ch;
}

static unsigned char pm_getrawbyte(FILE *file) {
  register int iby;
  
  iby = getc( file );
  if ( iby == EOF )
    pm_error("EOF / read error");
  return (unsigned char) iby;
}
