/* pixio.h, header fuer io-routinen fuer pixel */
#ifndef _PIXIO_H_
#define _PIXIO_H_

#include <stdlib.h>
#include <stdio.h>
#include "pixdef.h"

/* read_pbm
** liest ein RGB-Bitmap ein und gibt
** ein Feld von RGB-Alpha-Werten zurueck,
** wobei Alpha auf 0 gesetzt ist.
** erkannt werden netpbm P3- und P6-Formate
*/
extern pixval *read_pbm(
			     FILE *ifp, 
			     int *cols, int *rows, 
			     pixval *maxval);
/* read_rgba
** liest ein RGBA-Bitmap ein
** Magic Number : P7 oder P8
*/
extern pixval *read_rgba(
			     FILE *ifp, 
			     int *cols, int *rows, 
			     pixval *maxval);
/* read_pgm
** liest ein Greymap ein und gibt
** ein Feld von Greyvals zurueck,
** erkannt werden netpbm P2- und P5-Formate
*/
extern pixval *read_pgm(
			     FILE *ifp, 
			     int *cols, int *rows, 
			     pixval *maxval);

/* write_pbm
** kriegt ein Feld von RGB-Alpha-Werten und 
** schreibt eine P3-Bitmap, wobei Alpha
** ignoriert wird.
*/
extern void write_pbm(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);
/* write_pbm_raw
** kriegt ein Feld von RGB-Alpha-Werten und 
** schreibt eine P6-Bitmap, wobei Alpha
** ignoriert wird.
*/
extern void write_pbm_raw(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);

/* write_pgm
** kriegt ein Feld von Pixvals und 
** schreibt eine P2-Bitmap
*/
extern void write_pgm(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);
/* write_pgm_raw
** kriegt ein Feld von Pixvals und 
** schreibt eine P5-Bitmap
*/
extern void write_pgm_raw(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);
/* write_rgba
** kriegt ein Feld von RGB-Alpha-Werten und 
** schreibt es formatiert auf die Platte
** Magic Number: P7
*/
extern void write_rgba(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);
/* write_rgba
** kriegt ein Feld von RGB-Alpha-Werten und 
** schreibt es als Chars auf die Platte
** Magic Number: P8
*/
extern void write_rgba_raw(
		    FILE *ifp, 
		    pixval *pixels, 
		    int cols, int rows, 
		    pixval maxval);

/* pm_error
** gibt eine Fehlermeldung aus
** und bricht das Programm mit 1 ab
*/
extern void pm_error(char *reason);

#endif /*_PIXIO_H_*/






